package cz.fi.muni.pa165.dao;

import org.springframework.data.repository.NoRepositoryBean;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;
import java.util.List;
import java.util.UUID;

/**
 * @author Dominik Kozubík
 */
@NoRepositoryBean
public class BaseDaoImpl<U> implements BaseDao<U> {

    final Class<U> typeParameterClass;

    @PersistenceContext
    protected EntityManager em;

    public BaseDaoImpl(Class<U> typeParameterClass) {
        this.typeParameterClass = typeParameterClass;
    }

    @Override
    public void create(U u) {
        em.persist(u);
    }

    @Override
    public List<U> findAll() {

        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<U> cq = cb.createQuery(typeParameterClass);
        Root<U> rootEntry = cq.from(typeParameterClass);
        CriteriaQuery<U> all = cq.select(rootEntry);
        TypedQuery<U> allQuery = em.createQuery(all);
        return allQuery.getResultList();
    }

    @Override
    public U findById(UUID id) {
        return em.find(typeParameterClass, id);
    }

    @Override
    public void remove(U u) {
        em.remove(u);
    }
}
